import { CoreInstance, CoreOptions } from './wasm';
import { ImageLike, ImageFormat } from './image';
export interface LtasgOptions {
    core?: Readonly<CoreOptions>;
    /** The input image size. */
    imageSize: number;
    /**
     * The σ (standard deviation) value for each generated mip level.
     *
     * Large values (> 0.2) significantly increase the processing time in the
     * order of `O(σ²)` since the assumption that LTASG makes is no longer valid
     * under large σ values and it has to realize a large blur by a repeated
     * application of small-sized blurs.
     */
    mipLevelSigmas: ArrayLike<number>;
    /**
     * The minimum number of passes. Specify `1` for best performance, and
     * `2` or `3` for best quality. A larger value usually hurts both of
     * performance and quality.
     *
     * Defaults to `2`.
     */
    minNumPasses?: number;
    /**
     * (Advanced parameter) The resolution of the kernel. Defaults to `2`.
     */
    kernelResolution?: number;
    /**
     * (Advanced parameter) Specifies the size of the Gaussian kernel by the
     * ratio to the σ value. Defaults to `3`.
     */
    kernelWidth?: number;
}
/**
 * Provides a linear-time approximate spherical Gaussian blur algorithm.
 */
export declare class LtasgBlur {
    readonly core: CoreInstance;
    readonly size: number;
    private readonly plan;
    /**
     * Asynchronous constructor of `LtasgBlur`.
     */
    static create(options: Readonly<LtasgOptions>): Promise<LtasgBlur>;
    /**
     * Synchronous constructor of `LtasgBlur`. `options.core`,
     * `options.core.module` and `options.core.instance` must not be `null` or
     * `undefined` if this constructor is called from the main thread.
     */
    constructor(options: Readonly<LtasgOptions>);
    /**
     * Apply the blur on the cube map image.
     *
     * @return The processed mipmapped cube map image.
     */
    process(input: ArrayLike<Uint8Array | Float32Array | ImageLike>, inFormat: ImageFormat, outFormat: ImageFormat): Float32Array[][] | Uint8Array[][];
}
