"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Copyright (c) 2017 yvt
 *
 * This file is a part of hyper3d-envmapgen. Please read the license text that
 * comes with the source code for use conditions.
 */
var wasm_1 = require("./wasm");
var image_1 = require("./image");
var utils_1 = require("./utils");
function generateGaussianKernel(radius, sigma) {
    var v = new Float32Array(radius * 2 + 1);
    var sum = 0;
    for (var i = 0; i <= radius * 2; ++i) {
        sum += (v[i] = Math.exp(-0.5 * Math.pow(((i - radius) / sigma), 2)));
    }
    // normalize
    var scale = 1 / sum;
    for (var i = 0; i < v.length; ++i) {
        v[i] *= scale;
    }
    return v;
}
/**
 * Provides a linear-time approximate spherical Gaussian blur algorithm.
 */
var LtasgBlur = /** @class */ (function () {
    /**
     * Synchronous constructor of `LtasgBlur`. `options.core`,
     * `options.core.module` and `options.core.instance` must not be `null` or
     * `undefined` if this constructor is called from the main thread.
     */
    function LtasgBlur(options) {
        this.plan = [];
        this.core = new wasm_1.CoreInstance(options.core);
        this.size = options.imageSize | 0;
        var minNumPasses = (options.minNumPasses || 2) | 0;
        var kernelWidth = options.kernelWidth || 3;
        var kernelResolution = options.kernelResolution || 2;
        var lastVariance = 0;
        var levels = options.mipLevelSigmas;
        for (var i = 0; i < levels.length; ++i) {
            var size = (this.size + (1 << i) - 1) >> i;
            var desiredSigma = levels[i];
            var desiredVar = desiredSigma * desiredSigma;
            var residueVar = desiredVar - lastVariance;
            if (residueVar < 0) {
                throw new Error("mipLevelSigmas must be a monotonically increasing sequence");
            }
            lastVariance = desiredVar;
            // Upper bound of blur amount that can be applied by a single run of
            // `ltasg_single(..., {0, 1, 2}, ...)`
            var sigmaLimit = 0.5 / kernelWidth;
            var numPasses = Math.max(Math.ceil(residueVar / (sigmaLimit * sigmaLimit)), minNumPasses);
            var levelSigma = Math.sqrt(residueVar / numPasses) * size;
            var kernelRadius = Math.floor(levelSigma * kernelResolution * kernelWidth);
            var kernelScale = 1 / kernelResolution;
            var kernel = generateGaussianKernel(kernelRadius, levelSigma * kernelResolution);
            this.plan.push({
                kernel: kernel,
                kernelScale: kernelScale,
                numPasses: numPasses,
            });
        }
    }
    /**
     * Asynchronous constructor of `LtasgBlur`.
     */
    LtasgBlur.create = function (options) {
        return __awaiter(this, void 0, void 0, function () {
            var core;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, wasm_1.CoreInstance.create(options.core)];
                    case 1:
                        core = _a.sent();
                        return [2 /*return*/, new LtasgBlur(__assign({}, options, { core: core }))];
                }
            });
        });
    };
    /**
     * Apply the blur on the cube map image.
     *
     * @return The processed mipmapped cube map image.
     */
    LtasgBlur.prototype.process = function (input, inFormat, outFormat) {
        if (input.length < 6) {
            throw new Error("input.length must be ≥ 6");
        }
        var _a = this, core = _a.core, plan = _a.plan, size = _a.size;
        // Coerce to `RgbaF32PremulAlpha`
        var baseLevel = utils_1.table(6, function (i) {
            var img = input[i];
            var f32 = image_1.coerceToRgbaF32PremulAlphaFrom(img, inFormat, size, size);
            // Make it an unique object
            if (f32 === img) {
                f32 = new Float32Array(f32);
            }
            if (f32.length < size * size * 4) {
                throw new Error("One of the input images is too small");
            }
            return f32;
        });
        // Generate levels
        var levels = [];
        var currentLevel = baseLevel;
        var currentSize = size;
        var _loop_1 = function (i) {
            var newSize = currentSize;
            if (i > 0) {
                newSize = (newSize + 1) >> 1;
                // Resample from the previous mip level
                currentLevel = currentLevel.map(function (image) { return image_1.resampleRgbF32(image, currentSize, currentSize, newSize, newSize); });
            }
            currentSize = newSize;
            var _a = plan[i], kernel = _a.kernel, kernelScale = _a.kernelScale, numPasses = _a.numPasses;
            core.ltasg(currentLevel, currentLevel, currentSize, image_1.ImageFormat.RgbaF32PremulAlpha, kernel, kernelScale, numPasses);
            if (outFormat != image_1.ImageFormat.RgbaF32PremulAlpha) {
                levels.push(currentLevel.map(function (image) { return image_1.coerceRgbaF32PremulAlphaTo(image, outFormat); }));
            }
            else {
                levels.push(currentLevel);
            }
        };
        for (var i = 0; i < plan.length; ++i) {
            _loop_1(i);
        }
        return levels;
    };
    return LtasgBlur;
}());
exports.LtasgBlur = LtasgBlur;
//# sourceMappingURL=ltasgblur.js.map